/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "mptkcodec.h"

/**************************************************/
/* MAIN                                           */
/**************************************************/
int main( int argc, char **argv )
{
	/* Declarations */
	book_decoder *decoder;
	unsigned int n,numBytes,maxNumBytes, zeros;
	FILE *fid;

	/* Check number of arguments */
	if ( argc<5 ) {
		printf("Needs 4 arguments: [inputfile] [bitrate in bps] [mode] [outputfile] [zero padding (optional)]\n");
		return 1;
	}

	/* Initialize decoder */
	decoder = new book_decoder( atoi(argv[3]) );

	/* Open file */
	fid = fopen(argv[1],"r");

	/* Read Header */
	printf("Read header...");
	numBytes = decoder->read_header( fid );
	printf("%u bytes\n",numBytes);

	/* Read data */
	for ( n=0 ; n<decoder->numSlots ; n++ ) {
		printf("Read slot %u/%u...",n+1,decoder->numSlots);
		fread(&maxNumBytes,sizeof(unsigned int),1,fid);
		numBytes = decoder->read_slot(fid,atoi(argv[2]),maxNumBytes);
		decoder->decode_slot( n+decoder->firstNonZeroSlot, numBytes );
		fseek(fid,maxNumBytes-numBytes,SEEK_CUR);
		printf("%u bytes\n",numBytes);
	}

	/* Save output */
	if (atoi(argv[3])==0) {
		/* Threshold */
		for ( n=0 ; n<decoder->signal->numSamples ; n++ ) {
			if ( decoder->signal->storage[n]>1 ) decoder->signal->storage[n] = 1;
			if ( decoder->signal->storage[n]<-1 ) decoder->signal->storage[n] = -1;
		}
		if (EDGES_ZEROS) {
			zeros = 2*WIN_LEN_MAX;
			if (argc==6) zeros = atoi( argv[5] );
			MP_Signal_c *signal2 = MP_Signal_c::init( decoder->signal->numChans, decoder->signal->numSamples-2*zeros, decoder->signal->sampleRate );
			memcpy( signal2->storage, decoder->signal->storage+zeros, signal2->numSamples*sizeof(MP_Sample_t) );
			signal2->wavwrite( argv[4] );
			delete( signal2 );
		} else {
			decoder->signal->wavwrite( argv[4] );
		}
	}
	if (atoi(argv[3])==1) {
		FILE *fid2 = fopen( argv[4], "w" );
		decoder->book->print( fid2, MP_BINARY);
		fwrite( &decoder->numSamples, sizeof(long unsigned int), 1, fid2);
		fclose( fid2 );
	}

	/* Clean */
	delete( decoder );
	fclose( fid );
	fftw_cleanup();

	return 0;
}
