/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <sys/time.h>
#include <string.h>
#include "mptkcodec.h"

int main( int argc, char **argv )
{
	/* Declarations */
	long unsigned int win_len, zeros;
	MP_Signal_c *signal;
	MP_Dict_c *dict;
	MP_Block_c *block;
	MP_Book_c *book;
	MP_Mpd_Core_c *mp;
	FILE *fid;
	timeval tim;

	/* Check number of arguments */
	if ( argc<3 ) {
		printf("Needs at least 2 arguments: [wavfile] [bookfile] [snr in dB (optional)] [zero padding (optional)]\n");
		return 1;
	}

	/* Load Signal */
	printf("Load Signal...\n");
	if (EDGES_ZEROS) {
		MP_Signal_c *signal2 = MP_Signal_c::init( argv[1] );
		zeros = 2*WIN_LEN_MAX;
		if (argc==5) zeros = atoi( argv[4] );
		signal = MP_Signal_c::init(signal2->numChans, signal2->numSamples+2*zeros, signal2->sampleRate);
		memcpy( signal->storage+zeros, signal2->storage, signal2->numSamples*sizeof(MP_Sample_t) );
		signal->refresh_energy();
		delete( signal2 );
	} else {
		signal = MP_Signal_c::init( argv[1] );
	}
	printf("Load Signal...OK\n");

	/* Build dictionary */
	printf("Build dictionary...\n");
	dict = MP_Dict_c::init();
	win_len = WIN_LEN_MIN;
	while ( win_len<=WIN_LEN_MAX ) {
		if (win_len>PEC3) {
			block = MP_Mdct_Block_c::init( signal, win_len, WIN_TYPE, WIN_OPTIONAL, win_len>>2, PEC1, PEC2, PEC3 );
		} else {
			block = MP_Mdct_Block_c::init( signal, win_len, WIN_TYPE, WIN_OPTIONAL, win_len>>2, 0, 0, 0 );
		}
		dict->add_block( block );
		win_len = win_len << 1;
	}
	printf("Build dictionary...OK\n");

	/* Init Book */
	book = MP_Book_c::init();
	book->sampleRate = signal->sampleRate;

	/* Init MP */
	mp = MP_Mpd_Core_c::init( signal, book, MPD_NO_APPROXIMANT, dict );
	if (argc>3) {
		mp->set_snr_condition( atoi(argv[3]) );
	} else {
		mp->set_snr_condition( SNR );
	}
	mp->set_report_hit( MPD_REPORT_HIT );

	/* Run MP */
	printf("Run MP...\n");
	gettimeofday(&tim, NULL);
	double t1=tim.tv_sec+(tim.tv_usec/1000000.0);
	mp->run();
	gettimeofday(&tim, NULL);
	double t2=tim.tv_sec+(tim.tv_usec/1000000.0);
	printf("Run MP...OK %.6f s\n",t2-t1);

	/* Save book */
	printf("Save book...\n");
	fid = fopen(argv[2],"w");
	book->print(fid,MP_BINARY);
	fwrite( &signal->numSamples, sizeof(long unsigned int), 1, fid);
	fclose(fid);
	printf("Save book...OK\n");

	/* Clean */
	delete( mp );
	fftw_cleanup();

	return 0;
}
