/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <sys/time.h>
#include <string.h>
#include "mptkcodec.h"

int main( int argc, char **argv )
{
	/* Declarations */
	long unsigned int win_len, zeros, numBytes, n;
	MP_Signal_c *signal;
	MP_Dict_c *dict;
	MP_Block_c *block;
	MP_Book_c *book;
	MP_Mpd_Core_c *mp;
	FILE *fid;
	timeval tim;
	book_coder *coder;

	/* Check number of arguments */
	if ( argc!=7 ) {
		printf("Needs 6 arguments: [wavfile] [codedfile] [bitrate in bps] [mode] [snr in dB] [zero padding]\n");
		return 1;
	}

	/* Load Signal */
	printf("Load Signal...\n");
	MP_Signal_c *signal2 = MP_Signal_c::init( argv[1] );
	zeros = atoi( argv[6] );
	signal = MP_Signal_c::init(signal2->numChans, signal2->numSamples+2*zeros, signal2->sampleRate);
	memcpy( signal->storage+zeros, signal2->storage, signal2->numSamples*sizeof(MP_Sample_t) );
	signal->refresh_energy();
	delete( signal2 );
	printf("Load Signal...OK\n");

	/* Build dictionary */
	printf("Build dictionary...\n");
	dict = MP_Dict_c::init();
	win_len = WIN_LEN_MIN;
	while ( win_len<=WIN_LEN_MAX ) {
		if (win_len>PEC3) {
			block = MP_Mdct_Block_c::init( signal, win_len, WIN_TYPE, WIN_OPTIONAL, win_len>>2, PEC1, PEC2, PEC3 );
		} else {
			block = MP_Mdct_Block_c::init( signal, win_len, WIN_TYPE, WIN_OPTIONAL, win_len>>2, 0, 0, 0 );
		}
		dict->add_block( block );
		win_len = win_len << 1;
	}
	printf("Build dictionary...OK\n");

	/* Init Book */
	book = MP_Book_c::init();
	book->sampleRate = signal->sampleRate;

	/* Init MP */
	mp = MP_Mpd_Core_c::init( signal, book, MPD_NO_APPROXIMANT, dict );
	mp->set_snr_condition( atoi(argv[5]) );
	mp->set_report_hit( MPD_REPORT_HIT );

	/* Run MP */
	printf("Run MP...\n");
	gettimeofday(&tim, NULL);
	double t1=tim.tv_sec+(tim.tv_usec/1000000.0);
	mp->run();
	gettimeofday(&tim, NULL);
	double t2=tim.tv_sec+(tim.tv_usec/1000000.0);
	printf("Run MP...OK %.6f s\n",t2-t1);

	/* Open output file */
	fid = fopen(argv[2],"w");

	/* Initialize coder */
	printf("Run coding...\n");
	coder = new book_coder(book,signal->numSamples,atoi(argv[4]));
	coder->init_bitplane();

	/* Write Header */
	numBytes = coder->write_header( fid );

	/* Write data */
	for ( n=0 ; n<coder->numSlots ; n++ ) {
		coder->load_slot( coder->firstNonZeroSlot+n );
		coder->interleave_slot();
		numBytes = coder->code_slot( atoi(argv[3]) );
		fwrite(&numBytes,sizeof(unsigned int),1,fid );
		coder->write_slot( fid );
	}
	printf("Run coding...OK\n");

	/* Clean */
	delete( mp );
	delete( coder );
	fclose( fid );
	fftw_cleanup();

	return 0;
}
