/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mptkcodec.h"

bitstream::bitstream( unsigned int set_max_numBytes ) {

	max_numBytes = set_max_numBytes;
	bytes = new unsigned char[max_numBytes];
	maxNumBits = max_numBytes*8;
	reset();
}

bitstream::~bitstream( void ) {

	delete[]( bytes );
}

void bitstream::set_maxNumBits( unsigned int setmaxnumbits ) {

	if ( setmaxnumbits < max_numBytes*8 ) maxNumBits = setmaxnumbits;
}

void bitstream::reset( void ) {

	numBytes = 0;
	numBits = 0;
	buffer = 0;
	counter = 0;
}

int bitstream::put_bit( int bit ) {

	if (numBits+1>maxNumBits) return 1;
	buffer = buffer + (bit << counter);
	counter++;
	numBits++;
	if (counter==8) {
		bytes[numBytes] = buffer;
		numBytes++;
		buffer = 0;
		counter = 0;
	}
	return 0;
}

void bitstream::flush( void ) {

	if (counter>0) {
		bytes[numBytes] = buffer;
		numBytes++;
		buffer = 0;
		counter = 0;
	}
}

int bitstream::get_bit( int *bit ) {

	if (numBits+1>maxNumBits) return 1;
	if (counter==0) {
		buffer = bytes[numBytes];
		numBytes++;
	}
	*bit = buffer%2;
	buffer = buffer >> 1;
	counter++;
	numBits++;
	if (counter==8) {
		counter = 0;
	}
	return 0;
}

int bitstream::put_unsigned( unsigned int x, unsigned int n) {

	if (numBits+n>maxNumBits) return 1;
	for ( unsigned int m=0 ; m<n ; m++ ) {
		put_bit(x%2);
		x = x >> 1;
	}
	return 0;
}

int bitstream::get_unsigned( unsigned int *x, unsigned int n ) {

	if (numBits+n>maxNumBits) return 1;
	int bit = 0;
	*x = 0;
	for ( unsigned int m=0 ; m<n ; m++ ) {
		get_bit(&bit);
		*x = *x + (bit<<m);
	}
	return 0;
}
