/******************************************************************************/
/*                                                                            */
/*                               mtrand.h                                     */
/*                                                                            */
/*                        Matching Pursuit Library                            */
/*                                                                            */
/* Rmi Gribonval                                                             */
/* Sacha Krstulovic                                           Mon Feb 21 2005 */
/* -------------------------------------------------------------------------- */
/*                                                                            */
/*  Copyright (C) 2005 IRISA                                                  */
/*                                                                            */
/*  This program is free software; you can redistribute it and/or             */
/*  modify it under the terms of the GNU General Public License               */
/*  as published by the Free Software Foundation; either version 2            */
/*  of the License, or (at your option) any later version.                    */
/*                                                                            */
/*  This program is distributed in the hope that it will be useful,           */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
/*  GNU General Public License for more details.                              */
/*                                                                            */
/*  You should have received a copy of the GNU General Public License         */
/*  along with this program; if not, write to the Free Software               */
/*  Foundation, Inc., 59 Temple Place - Suite 330,                            */
/*  Boston, MA  02111-1307, USA.                                              */
/*                                                                            */
/******************************************************************************/

/* ---------------------------------------------- */
/* ------ Mersenne-Twister random generator ----- */
/* ---------------------------------------------- */
/*
   A C-program for MT19937, with initialization improved 2002/1/26.
   Coded by Takuji Nishimura and Makoto Matsumoto.

   Hacked 2006/1/31 by Sacha for inclusion in mptk.

   Before using, initialize the state by using init_genrand(seed)
   or init_by_array(init_key, key_length).

   Copyright (C) 1997 - 2002, Makoto Matsumoto and Takuji Nishimura,
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

     1. Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.

     2. Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.

     3. The names of its contributors may not be used to endorse or promote
        products derived from this software without specific prior written
        permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

   Any feedback is very welcome.
   http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/emt.html
   email: m-mat @ math.sci.hiroshima-u.ac.jp (remove space)
*/

#ifndef _mtrand_h_
# define _mtrand_h_

#ifdef __cplusplus
extern "C" {
#endif

  /* Period parameters */
#define MTRAND_N 624
#define MTRAND_M 397
#define MTRAND_MATRIX_A 0x9908b0dfUL   /* constant vector a */
#define MTRAND_UPPER_MASK 0x80000000UL /* most significant w-r bits */
#define MTRAND_LOWER_MASK 0x7fffffffUL /* least significant r bits */

  /* Mersenne twister random generator */
  /* initializes the random generator with a seed */
  void init_genrand(unsigned long s);
  /* generates a random number on [0,0xffffffff]-interval */
  unsigned long genrand_int32(void);
  /* generates a random number on [0,1]-real-interval */
#define genrand_real1()  ( ((float)(genrand_int32())) / 4294967295.0 ) /* divided by 2^32-1 */
  /* generates a random number on [0,1[-real-interval */
#define genrand_real2()  ( (((float)(genrand_int32())) / 4294967296.0 ) /* divided by 2^32 */
  /* generates a random number on ]0,1[-real-interval */
#define genrand_real3()  ( ((float)(genrand_int32())) + 0.5) / 4294967296.0 ) /* shifted,
										 then divided by 2^32 */
  /** \brief A normal (Gaussian) random generator based on the Mersenne-Twister
   * uniform random generator and the polar form of the Box-Muller tranform.
   *
   *  \param mean the mean of the random variable
   *  \param var the variance of the random variable
   *  \return a Gaussian random value
   */
  double mt_nrand( double mean, double var );

#ifdef __cplusplus
}
#endif

#endif /* _mtrand_h_ */
