/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <sys/time.h>
#include <string.h>
#include <math.h>
#include "mptkcodec.h"

int main( int argc, char **argv )
{
	/* Declarations */
	FILE *fid;
	MP_Book_c *book;
	unsigned int numSamples,n, zeros;
	book_coder *coder;

	/* Check number of arguments */
	if ( argc<3 ) {
		printf("Needs 2 arguments: [bookfile] [wavfile] [zero padding (optional)]\n");
		return 1;
	}

	/* Load book and number of Samples */
	book =  MP_Book_c::init();
	fid = fopen(argv[1],"r");
	book->load( fid );
	fread(&numSamples,sizeof(long unsigned int),1,fid);
	fclose(fid);

	/* Init book coder */
	coder = new book_coder( book, numSamples, 0 );
	coder->init_signal();
	coder->init_mdct();

	/* Synthesize book */
	for ( n=coder->firstNonZeroSlot ; n<=coder->lastNonZeroSlot ; n++ ) {
		coder->load_slot( n );
		coder->synth_slot( n );
	}

	/* Threshold */
	for ( n=0 ; n<coder->signal->numSamples ; n++ ) {
		if ( coder->signal->storage[n]>1 ) coder->signal->storage[n] = 1;
		if ( coder->signal->storage[n]<-1 ) coder->signal->storage[n] = -1;
	}

	/* Output to wav */
	if (EDGES_ZEROS) {
		zeros = 2*WIN_LEN_MAX;
		if (argc==4) zeros = atoi( argv[3] );
		MP_Signal_c *signal2 = MP_Signal_c::init( coder->signal->numChans, coder->signal->numSamples-2*zeros, coder->signal->sampleRate );
		memcpy( signal2->storage, coder->signal->storage+zeros, signal2->numSamples*sizeof(MP_Sample_t) );
		signal2->wavwrite( argv[2] );
		delete( signal2 );
	} else {
		coder->signal->wavwrite( argv[2] );
	}

	/* Clean */
	delete( book );
	delete( coder );

	return 0;
}
